/**************************************************************************//**
*   @file   mb_interrupts.c
*   @brief  Interrupts handling functions driver file.
*   @author ATofan (alexandru.tofan@analog.com)
*
*******************************************************************************
* Copyright 2011(c) Analog Devices, Inc.
*
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without modification,
* are permitted provided that the following conditions are met:
*  - Redistributions of source code must retain the above copyright
*    notice, this list of conditions and the following disclaimer.
*  - Redistributions in binary form must reproduce the above copyright
*    notice, this list of conditions and the following disclaimer in
*    the documentation and/or other materials provided with the
*    distribution.
*  - Neither the name of Analog Devices, Inc. nor the names of its
*    contributors may be used to endorse or promote products derived
*    from this software without specific prior written permission.
*  - The use of this software may or may not infringe the patent rights
*    of one or more patent holders.  This license does not release you
*    from the requirement that you obtain separate licenses from these
*    patent holders to use this software.
*  - Use of the software either in source or binary form, must be run
*    on or directly connected to an Analog Devices Inc. component.
*
* THIS SOFTWARE IS PROVIDED BY ANALOG DEVICES "AS IS" AND ANY EXPRESS OR IMPLIED
* WARRANTIES, INCLUDING, BUT NOT LIMITED TO, NON-INFRINGEMENT, MERCHANTABILITY
* AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
* IN NO EVENT SHALL ANALOG DEVICES BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
* SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
* INTELLECTUAL PROPERTY RIGHTS, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
* LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
* ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
* (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
* SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*
*******************************************************************************
*   SVN Revision: $WCREV$
******************************************************************************/

#include <stdio.h>
#include "mb_interrupts.h"

extern volatile int  rxData;
extern volatile int  timeStepValue;
extern volatile int  dacStepValue;
extern volatile int  dacStep;
extern volatile char dacTop;
extern volatile char dacBot;
extern volatile char dacSelected;

/******************************************************************************
* @brief UART Interrupt Handler.
*
* @param None.
*
* @return None.
******************************************************************************/
void uartIntHandler(void)
{
	// If UART has triggered a RX DATA Interrupt
	// store the read value in rxData
	if(Xil_In32(UART_BASEADDR + 0x08) & 0x01)
	{
		// Save read data from UART
		rxData = Xil_In32(UART_BASEADDR);
	}
}

/******************************************************************************
* @brief Timer Interrupt Handler.
*
* @param None.
*
* @return None.
******************************************************************************/
void timerIntHandler(void)
{
	int prevDacStep = dacStep;
	// Write Load Register with value to count to
	Xil_Out32(TIMER_BASEADDR + 0x04, timeStepValue);
	// Increase output data value
	if(dacBot == 1)
	{
		dacStep = dacStep + dacStepValue;
	}
	else if(dacTop == 1)
	{
		dacStep = dacStep - dacStepValue;
	}
	if((dacStep > 0xFFF) && (dacTop == 0))
	{
		dacTop = 1;
		dacBot = 0;
	}
	if((dacStep < 0x000) && (dacBot == 0))
	{
		dacTop = 0;
		dacBot = 1;
	}
	if(dacStep > 0xFFF)
	{
		dacStep = 0xFFF;
	}
	if(dacStep < 0x000)
	{
		dacStep = 0x000;
	}

	if(dacStep != prevDacStep)
	{
		AD5628_WriteUpdateDac(dacSelected, dacStep); // Load the DAC with desired value
	}
	Xil_Out32(TIMER_BASEADDR + 0x00, 0x00000060);
	// Clear Timer Interrupts
	Xil_Out32(TIMER_BASEADDR + 0x00, 0x00000100);
	Xil_Out32(TIMER_BASEADDR + 0x00, 0x000001C0);
}

/******************************************************************************
* @brief Enable Interrupts.
*
* @param None.
*
* @return None.
******************************************************************************/
void enableInterrupts(void)
{
	// Register UART Interrupt Service Routine
	XIntc_RegisterHandler(INTC_BASEADDR, UART_INTERRUPT, (XInterruptHandler)uartIntHandler, (void *) 0);

	// Register TIMER0 Interrupt Service Routine
	XIntc_RegisterHandler(INTC_BASEADDR, TIMER_INTERRUPT, (XInterruptHandler)timerIntHandler, (void *) 0);

	// Enable Interrupt Controller
	XIntc_MasterEnable(INTC_BASEADDR);
	XIntc_EnableIntr(INTC_BASEADDR, UART_INTERRUPT_MASK | TIMER_INTERRUPT_MASK);

	// Enable UART Interrupts
	Xil_Out32(UART_BASEADDR + 0x0C, (1 << 4));

	// Enable TIMER Interrupts and Initial Settings
	Xil_Out32(TIMER_BASEADDR + 0x04, timeStepValue);
	Xil_Out32(TIMER_BASEADDR + 0x00, 0x00000060);

	// Enable MicroBlaze Interrupts
	microblaze_enable_interrupts();
}
